using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.PostProcessing;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MVR.FileManagementSecure;
using System.Linq;

namespace Feel
{
    public class VendingMachineLite : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel"; // edited by VamTastic (original by Frief )
        public string pluginName = "VendingMachineLite";
        public string pluginVersion = "1";
        public string pluginDate = "07/02" + "/2023";
        public string pluginDescription = @"Used to open and close the doors";
        #endregion

        #region Vars    

        private Atom _thisAtom;
        private FreeControllerV3 _thisControl;
        private Rigidbody camerarig = null;

        private Transform ObjectRoot;
        private Transform MainLowRoot;

        private string currType = "Opaque";

        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        #endregion

        // IMPORTANT - DO NOT make custom enums. The dynamic C# complier crashes Unity when it encounters these for
        // some reason

        // IMPORTANT - DO NOT OVERRIDE Awake() as it is used internally by MVRScript - instead use Init() function which
        // is called right after creation
        public override void Init()
        {
            try
            {
                _thisAtom = containingAtom;
                _thisControl = _thisAtom.freeControllers.First(fc => fc.name == "control");

                List<string> uids = SuperController.singleton.GetAtomUIDs();

                foreach (string uid in uids)
                {
                    if(uid == "[CameraRig]")
                    {
                        Atom atom = SuperController.singleton.GetAtomByUid(uid);
                        //FreeControllerV3 thisControl = atom.freeControllers.First(fc => fc.name == "control");

                        //Vector3 _dir = (lastInput_ - transform.position).normalized;

                        //SuperController.LogMessage(">> " + atom.rigidbodies);
                        foreach(Rigidbody body in atom.rigidbodies)
                        {
                            //SuperController.LogMessage(">> " + body.name);
                            if(body.name == "CenterEye")
                            {
                                //float angle = Vector3.Angle(_thisControl.transform.position, body.transform.position);

                                //SuperController.LogMessage(">> " + angle);
                                camerarig = body;
                                break;
                            }
                        }

                        break;
                    }
                }

                // put init code in here
                //SuperController.LogMessage("Template Loaded");
                JSONStorableAction reset_action = new JSONStorableAction("GoToSupport", OnGoToSupport);
                RegisterAction(reset_action);

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Vending Machine</b></size></color>\n\n" +
                    "<color=#333>" +
                    "store the position of the target,\n" +
                    "or Track a specific target.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

                var btn = CreateButton("Go To Support", true);
                btn.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

                // create custom JSON storable params here if you want them to be stored with scene JSON
                // types are JSONStorableFloat, JSONStorableBool, JSONStorableString, JSONStorableStringChooser
                // JSONStorableColor

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Start is called once before Update or FixedUpdate is called and after Init()
        void Start()
        {
            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            try
            {
                if (initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading)
                {
                    initLoops++;
                    InitEditor();
                }

                if (initLoops == maxInitLoops)
                {
                    SuperController.LogError("GlassBody : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                    initLoops++; // To avoid spamming the log with the error
                }

                if(camerarig != null)
                {
                    float angle = Vector3.Angle(_thisControl.transform.position, camerarig.transform.position);
                    //SuperController.LogMessage(">> " + angle);
                    //Renderer renderer = MainLowRoot.GetComponent<Renderer>();

                    if (angle < 100f)
                    {
                        if(currType != "Transparent")
                        {
                            ChangeRenderMode(MainLowRoot, "Transparent");
                            currType = "Transparent";
                        }

                    }
                    else
                    {
                        if(currType != "Opaque")
                        {
                            ChangeRenderMode(MainLowRoot, "Opaque");
                            currType = "Opaque";
                        }
                    }

                }

                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // FixedUpdate is called with each physics simulation frame by Unity
        void FixedUpdate()
        {
            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy()
        {
        }

        private bool InitEditor()
        {
            if (editorInitialized == true) return true;

            ObjectRoot = getObjectRoot(containingAtom.reParentObject, "VendingMachineLite");
            if (ObjectRoot == null)
            {
                return false;
            }

            MainLowRoot = getChildRoot(containingAtom.reParentObject, "Main_Low");

            //SuperController.LogMessage(">> ----------------------------");

            // I found my transforms, now the rest
            if (ObjectRoot && MainLowRoot)
            {
                editorInitialized = true;

                //Renderer renderer = MainLowRoot.GetComponent<Renderer>();
                //Color color = Color.white;
                //color.a = 0.5f;
                //renderer.material.color = color;

                ChangeRenderMode(MainLowRoot, "Opaque");

                // Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
                // I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
                // the float val param of the callbacks is useless

                return true;
            }
            else
            {
                SuperController.LogError("VendingMachineLite : your CUA is missing mandatory components.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }

            return false;
        }

        private Transform getObjectRoot(Transform parent, string rootName)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(rootName))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child, rootName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private Transform getChildRoot(Transform parent, string searchName)
        {
            foreach (Transform child in parent)
            {
                if (child.name == searchName)
                {

                    return child;
                }
                else
                {
                    Transform childSearch = getChildRoot(child, searchName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private void OnGoToSupport()
        {
            Application.OpenURL("https://www.patreon.com/posts/78537735");
        }


        protected void ChangeRenderMode(Transform target, string blendMode)
        {
            if (target == null)
            {
                return;
            }
            Renderer renderer = target.GetComponent<Renderer>();

            if (blendMode == "Opaque")
            {
                renderer.material.SetFloat("_Mode", 0.0f);
                renderer.material.SetOverrideTag("RenderType", "Opaque");
                renderer.material.SetInt("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.One);
                renderer.material.SetInt("_DstBlend", (int)UnityEngine.Rendering.BlendMode.Zero);
                renderer.material.SetInt("_ZWrite", 1);
                renderer.material.DisableKeyword("_ALPHATEST_ON");
                renderer.material.DisableKeyword("_ALPHABLEND_ON");
                renderer.material.DisableKeyword("_ALPHAPREMULTIPLY_ON");
                renderer.material.renderQueue = -1;
            }
            else
            {
                renderer.material.SetFloat("_Mode", 3.0f);
                renderer.material.SetOverrideTag("RenderType", "Transparent");
                renderer.material.SetInt("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.One);
                renderer.material.SetInt("_DstBlend", (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);
                renderer.material.SetInt("_ZWrite", 0);
                renderer.material.DisableKeyword("_ALPHATEST_ON");
                renderer.material.DisableKeyword("_ALPHABLEND_ON");
                renderer.material.EnableKeyword("_ALPHAPREMULTIPLY_ON");
                renderer.material.renderQueue = 3000;
            }
        }

    }
	
}